import java.awt.Color;
import java.util.Vector;

import processing.core.PApplet;

public class ColorParam {

	int a;
	int r;
	int g;
	int b;

	public void setA(int a) {
		this.a = a;
	}

	public void setR(int r) {
		this.r = r;
	}

	public void setG(int g) {
		this.g = g;
	}

	public void setB(int b) {
		this.b = b;
	}

	ColorParam() {
		DefaultValues.loadColor(this);
	}

	ColorParam(byte a, byte r, byte g, byte b) {
		this.a = a;
		this.r = r;
		this.g = g;
		this.b = b;
	}

	ColorParam(int argb) {
		this.a = (int) (argb >> 24 & 0xFF);
		this.r = (int) (argb >> 16 & 0xFF);
		this.g = (int) (argb >> 8 & 0xFF);
		this.b = (int) (argb >> 0 & 0xFF);
	}

	public ColorParam(ColorParam color) {
		this.a = color.getA();
		this.r = color.getR();
		this.g = color.getG();
		this.b = color.getB();
	}

	public void setAHSB(byte a, float h, float s, float b) {
		this.a = a;
		this.r = (int) (Color.HSBtoRGB(h, s, b) >> 16 & 0xFF);
		this.g = (int) (Color.HSBtoRGB(h, s, b) >> 8 & 0xFF);
		this.b = (int) (Color.HSBtoRGB(h, s, b) >> 0 & 0xFF);
	}

	boolean equals(ColorParam col) {
		boolean equals = true;
		if (col.getA() != getA())
			equals = false;
		if (col.getR() != getR())
			equals = false;
		if (col.getG() != getG())
			equals = false;
		if (col.getB() != getB())
			equals = false;
		return equals;
	}

	int getA() {
		return a;
	}

	int getR() {
		return r;
	}

	int getG() {
		return g;
	}

	int getB() {
		return b;
	}

	int[] getARGB() {
		int[] color = { a, r, g, b };
		return color;
	}

	int getARGBInt() {
		int color = (a << 24) | (r << 16) | (g << 8) | (b << 0);
		return color;
	}

	float getHue() {
		return Color.RGBtoHSB(r, g, b, null)[0];
	}

	float getSaturation() {
		return Color.RGBtoHSB(r, g, b, null)[1];
	}

	float getBrightness() {
		return Color.RGBtoHSB(r, g, b, null)[2];
	}

	float[] getHSB() {
		return Color.RGBtoHSB(r, g, b, null);
	}

	public int getContrasted(float sC) {
		int color = (a << 24) | ((int) (r * sC) << 16) | ((int) (g * sC) << 8)
				| ((int) (b * sC) << 0);
		return color;
	}

	public void setARGB(byte a, byte r, byte g, byte b) {
		this.a = a;
		this.r = r;
		this.g = g;
		this.b = b;
	}

	public void randomSet() {
		a = (int) 0xFF;
		r = (int) Math.floor(Math.random() * 0xFF);
		g = (int) Math.floor(Math.random() * 0xFF);
		b = (int) Math.floor(Math.random() * 0xFF);
	}

	public void randomWalk() {
		a = (int) PApplet.constrain((int) ((int) a + (int) Math.floor(Math
				.random() * 0x22)), 0, 0xFF);
		r = (int) PApplet.constrain((int) ((int) r + (int) Math.floor(Math
				.random() * 0x22)), 0, 0xFF);
		g = (int) PApplet.constrain((int) (int) (g + (int) Math.floor(Math
				.random() * 0x22)), 0, 0xFF);
		b = (int) PApplet.constrain((int) ((int) b + (int) Math.floor(Math
				.random() * 0x22)), 0, 0xFF);
	}

	public int getShadowed(OutlineParam outline) {
		float contrast = outline.getShadowContrast();
		int color = (a << 24) | ((int) (r * contrast) << 16)
				| ((int) (g * contrast) << 8) | ((int) (b * contrast) << 0);
		return color;
	}

	public String[] getSaveStrings() {
		Vector<String> result = new Vector<String>();
		result.add(PApplet.str(a));
		result.add(PApplet.str(r));
		result.add(PApplet.str(g));
		result.add(PApplet.str(b));
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadSaveStrings(Vector<String> toLoad) {
		a = Integer.parseInt(toLoad.remove(0));
		r = Integer.parseInt(toLoad.remove(0));
		g = Integer.parseInt(toLoad.remove(0));
		b = Integer.parseInt(toLoad.remove(0));
		return toLoad;
	}

}
