import java.util.Vector;
import processing.core.PApplet;
import processing.core.PConstants;
import processing.core.PGraphics;

public class Constitution {

	public Vector<PatchableLimb> getPLimb() {
		return pLimb;
	}

	public void setPLimb(Vector<PatchableLimb> limb) {
		pLimb = limb;
	}

	public SpikesULimb getSpikes() {
		return spikes;
	}

	public void setSpikes(SpikesULimb spikes) {
		this.spikes = spikes;
	}

	public FeetULimb getFeet() {
		return feet;
	}

	public void setFeet(FeetULimb feet) {
		this.feet = feet;
	}

	public WingsULimb getWings() {
		return wings;
	}

	public void setWings(WingsULimb wings) {
		this.wings = wings;
	}

	public TummyULimb getTummy() {
		return tummy;
	}

	public void setTummy(TummyULimb tummy) {
		this.tummy = tummy;
	}

	PApplet parent;
	MainBody mainBody;

	Vector<PatchableLimb> pLimb;
	SpikesULimb spikes;
	FeetULimb feet;
	WingsULimb wings;
	TummyULimb tummy;

	Constitution(PApplet p, MainBody mainBody) {
		this.parent = p;
		this.mainBody = mainBody;
		this.pLimb = new Vector<PatchableLimb>();
		this.spikes = new SpikesULimb(parent);
		this.feet = new FeetULimb(parent);
		this.wings = new WingsULimb(parent);
		this.tummy = new TummyULimb(parent);
		spikes.setColor(mainBody.getColor());
		feet.setColor(mainBody.getColor());
		wings.setColor(mainBody.getColor());
		tummy.setColor(mainBody.getColor());
		spikes.setOutline(mainBody.getOutline());
		feet.setOutline(mainBody.getOutline());
		wings.setOutline(mainBody.getOutline());
		tummy.setOutline(mainBody.getOutline());
	}

	public PGraphics displayStill(int w, int h) {
		return displayAnim(w, h, -1, 0);
	}

	PGraphics render;

	public PGraphics displayAnim(int w, int h, float time, int anim) {
		render = parent.createGraphics(w, h, PConstants.JAVA2D);
		render.beginDraw();
		render.image(wings.displayAnim(w, h, time, anim), 0, 0);
		render.image(feet.displayAnim(w, h, time, anim), 0, 0);
		render.image(spikes.displayAnim(w, h, time, anim), 0, 0);
		render.image(tummy.displayAnim(w, h, time, anim), 0, 0);
		for (int i = 0; i < pLimb.size(); i++) {
			render.pushMatrix();
			doMovementFor(i, time, anim, w, h);
			render.image(pLimb.get(i).displayAnim(w, h, time, anim), 0, 0);
			render.popMatrix();
		}
		render.endDraw();
		return render;
	}

	private void doMovementFor(int i, float time, int anim, float w, float h) {
		// Do the scaling for the movement of a limb
		// negative numbers are unique limbs
		if (i >= 0) {
			float[] trans = pLimb.get(i).getMovement().getTranslationFor(anim,
					time);
			float x = trans[0];
			float y = trans[1];
			render.translate(x * w, y * h);
		}
	}

	public void addPatchable(int type) {
		// adds a patchable limb
		if (type == 0) {
			pLimb.add(new BeakPLimb(parent, this));
		}
		if (type == 1) {
			pLimb.add(new BlockPLimb(parent, this));
		}
		if (type == 2) {
			pLimb.add(new EyePLimb(parent, this));
		}
		if (type == 3) {
			pLimb.add(new TentaclePLimb(parent, this));
		}
		pLimb.get(pLimb.size() - 1).setColor(mainBody.getColor());
		pLimb.get(pLimb.size() - 1).setOutline(mainBody.getOutline());
	}

	public void extendParameters(ColorParam color, OutlineParam outline) {
		spikes.setColor(color);
		feet.setColor(color);
		wings.setColor(color);
		tummy.setColor(color);
		spikes.setOutline(outline);
		feet.setOutline(outline);
		wings.setOutline(outline);
		tummy.setOutline(outline);
		for (int i = 0; i < pLimb.size(); i++) {
			pLimb.get(i).setColor(color);
			pLimb.get(i).setOutline(outline);
		}
	}

	public void randomSet() {
		spikes.randomSet();
		feet.randomSet();
		wings.randomSet();
		tummy.randomSet();
		// random settings for unique limbs
		int rUL = PApplet.floor((float) (Math.random() * 5));
		if (rUL == 0) {
			spikes.setNumber(1);
			feet.setNumber(0);
			wings.setNumber(0);
			tummy.setNumber(0);
		} else if (rUL == 1) {
			spikes.setNumber(0);
			feet.setNumber(0);
			wings.setNumber(0);
			tummy.setNumber(1);
		} else if (rUL == 2) {
			spikes.setNumber(1);
			feet.setNumber((int) (Math.random() * 5));
			wings.setNumber(0);
			tummy.setNumber(0);
		} else if (rUL == 3) {
			spikes.setNumber(0);
			feet.setNumber(0);
			wings.setNumber((int) (Math.random() * 6));
			tummy.setNumber(1);
		} else if (rUL == 4) {
			spikes.setNumber(1);
			feet.setNumber((int) (Math.random() * 10));
			wings.setNumber(1);
			tummy.setNumber(0);
		}
		pLimb.clear();
		while (Math.random() < 0.5) {
			addPatchable(2);
		}
		while (Math.random() < 0.2) {
			addPatchable((int) Math.floor(Math.random() * 4));
		}
		for (int i = 0; i < pLimb.size(); i++) {
			pLimb.get(i).randomSet();
		}
	}

	public int getPLimbSize() {
		return pLimb.size();
	}

	public void clearLimbs() {
		spikes.setNumber(0);
		feet.setNumber(0);
		wings.setNumber(0);
		tummy.setNumber(0);
		pLimb.clear();
	}

	public void resetUniqueLimb(int i) {
		if (i == 0) {
			spikes = new SpikesULimb(parent);
		} else if (i == 1) {
			feet = new FeetULimb(parent);
		} else if (i == 2) {
			wings = new WingsULimb(parent);
		} else if (i == 3) {
			tummy = new TummyULimb(parent);
		}
	}

	public String[] getPLList() {
		// The list for interface buttons
		Vector<String> list = new Vector<String>();
		for (int i = 0; i < pLimb.size(); i++) {
			if (pLimb.get(i).type == 0) {
				list.add(i + " Beak");
			}
			if (pLimb.get(i).type == 1) {
				list.add(i + " Block");
			}
			if (pLimb.get(i).type == 2) {
				list.add(i + " Eye");
			}
			if (pLimb.get(i).type == 3) {
				list.add(i + " Tentacle");
			}
		}
		String[] list2 = new String[list.size()];
		for (int i = 0; i < list.size(); i++) {
			list2[i] = list.get(i);
		}
		return list2;
	}

	public String[] getSaveStrings() {
		Vector<String> result = new Vector<String>();
		result.add(PApplet.str(pLimb.size()));
		for (int i = 0; i < pLimb.size(); i++) {
			result.add(PApplet.str(pLimb.get(i).getType()));
			result = MyUtils.addStringArray(result, pLimb.get(i)
					.getSaveStrings());
		}
		result = MyUtils.addStringArray(result, spikes.getSaveStrings());
		result = MyUtils.addStringArray(result, feet.getSaveStrings());
		result = MyUtils.addStringArray(result, wings.getSaveStrings());
		result = MyUtils.addStringArray(result, tummy.getSaveStrings());
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadSaveStrings(Vector<String> toLoad) {
		int pLimbSize = Integer.parseInt(toLoad.remove(0));
		pLimb = new Vector<PatchableLimb>();
		for (int i = 0; i < pLimbSize; i++) {
			int type = Integer.parseInt(toLoad.remove(0));
			if (type == 0) {
				pLimb.add(new BeakPLimb(parent, this));
			}
			if (type == 1) {
				pLimb.add(new BlockPLimb(parent, this));
			}
			if (type == 2) {
				pLimb.add(new EyePLimb(parent, this));
			}
			if (type == 3) {
				pLimb.add(new TentaclePLimb(parent, this));
			}
			toLoad = pLimb.get(i).loadSaveStrings(toLoad);
		}
		toLoad = spikes.loadSaveStrings(toLoad);
		toLoad = feet.loadSaveStrings(toLoad);
		toLoad = wings.loadSaveStrings(toLoad);
		toLoad = tummy.loadSaveStrings(toLoad);
		return toLoad;
	}

}
