import processing.core.PApplet;
import processing.core.PConstants;

public class EyePLimb extends PatchableLimb {

	EyePLimb(PApplet p, Constitution constitution) {
		super(p, constitution);
		DefaultValues.loadEye(this);
		type = 2;		
	}	

	protected void drawStillRender() {
		float lParam = scaling.getL();
		render.fill(255);
		render.ellipse(0, 0, 1, 1);
		setColorVariance();
		render.noStroke();
		render.ellipse(0, 0, lParam, lParam);
	}

	protected void drawDownRender(float time) {
		float lParam = scaling.getL();
		render.fill(255);
		render.ellipse(0, 0, 1, 1);
		setColorVariance();
		render.noStroke();
		float y = (1 - lParam) * time / 2;
		render.ellipse(0, y, lParam, lParam);
	}

	protected void drawUpRender(float time) {
		float lParam = scaling.getL();
		render.fill(255);
		render.ellipse(0, 0, 1, 1);
		setColorVariance();
		render.noStroke();
		float y = -(1 - lParam) * time / 2;
		render.ellipse(0, y, lParam, lParam);
	}

	protected void drawLeftRender(float time) {
		float lParam = scaling.getL();
		render.fill(255);
		render.ellipse(0, 0, 1, 1);
		setColorVariance();
		render.noStroke();
		float x = (1 - lParam) / 2;
		render.ellipse(x, 0, lParam, lParam);
	}

	protected void drawLeftRightRender(float time) {
		float lParam = scaling.getL();
		render.fill(255);
		render.ellipse(0, 0, 1, 1);
		setColorVariance();
		render.noStroke();
		float x = (1 - lParam) * PApplet.sin(time * PConstants.TWO_PI) / 2;
		render.ellipse(x, 0, lParam, lParam);
	}

	protected void drawUpDownRender(float time) {
		float lParam = scaling.getL();
		render.fill(255);
		render.ellipse(0, 0, 1, 1);
		setColorVariance();
		render.noStroke();
		float y = -(1 - lParam) * PApplet.sin(time * PConstants.TWO_PI) / 2;
		render.ellipse(0, y, lParam, lParam);
	}

	@Override
	protected void drawFFlyingRender(float time) {
		drawLeftRightRender(time);
	}

	@Override
	protected void drawJAirRender(float time) {
		drawStillRender();
	}

	@Override
	protected void drawJCrounchRender(float time) {
		drawUpRender(time);
	}

	@Override
	protected void drawJgroundRender(float time) {
		drawDownRender(time);
	}

	@Override
	protected void drawWIdlingRender(float time) {
		drawLeftRightRender(time);
	}

	@Override
	protected void drawWJumpingRender(float time) {
		drawUpDownRender(time);
	}

	@Override
	protected void drawWWalkingRender(float time) {
		drawLeftRender(time);
	}

}
