import java.util.Vector;

import processing.core.PApplet;
import processing.core.PConstants;
import processing.core.PGraphics;

public class FeetULimb extends UniqueLimb {

	float convergeance;
	float base;

	public FeetULimb(PApplet p) {
		super(p);
		DefaultValues.loadFeet(this);
	}

	protected String[] getAdditionnalSaveStrings() {
		Vector<String> result = new Vector<String>();
		result.add(PApplet.str(convergeance));
		result.add(PApplet.str(base));
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadAdditionnalSaveStrings(Vector<String> toLoad) {
		convergeance = Float.parseFloat(toLoad.remove(0));
		base = Float.parseFloat(toLoad.remove(0));
		return toLoad;
	}

	public PGraphics displayAnim(float w, float h, float time, int anim) {
		beginRender(w, h);
		for (int i = 0; i < number; i++) {
			setColorVariance(i);
			render.noStroke();
			drawShape(anim, i, time);
		}
		for (int i = 0; i < number; i++) {
			setColorVariance(i);
			render.noFill();
			drawShapeWithoutShadow(anim, i, time);
		}
		endRender();
		return render;
	}

	protected void drawShape(int animType, int iter, float time) {
		drawShapeWithoutShadow(animType, iter, time);
		drawShadow(animType, iter, time);
	}

	protected void drawShapeWithoutShadow(int animType, int iter, float time) {
		render.beginShape();
		drawVertexes(switchAnim(animType, iter, time, 1));
		mainShape[0] = switchAnim(animType, iter, time, 1);
		render.endShape();
	}

	protected void drawShadow(int animType, int iter, float time) {
		setShadowColor(iter);
		render.pushMatrix();
		render.beginShape();
		// shifting of the shadow
		float xSh = (float) Math.cos(outline.getShadowPosition())
				* outline.getShadowDistance() / 3;
		float ySh = (float) Math.sin(outline.getShadowPosition())
				* outline.getShadowDistance() / 3;
		render.translate(xSh, ySh);
		drawVertexesShadow(switchAnim(animType, iter, time, outline
				.getShadowLevel()), mainShape[0], xSh, ySh);
		render.endShape();
		render.popMatrix();
	}

	Shape switchAnim(int animType, int iter, float time, float size) {
		if (animType == 0) {
			return drawShapeStill(size, iter);
		} else if (animType == 1) {
			return drawShapeWalking(size, time, iter);
		} else if (animType == 2) {
			return drawShapeStill(size, iter);
		} else if (animType == 3) {
			return drawShapeJumping(size, time, iter);
		} else if (animType == 4) {
			return drawShapeStill(size, iter);
		} else if (animType == 5) {
			return drawShapeCrounched(size, time, iter);
		} else if (animType == 6) {
			return drawShapeJumping(size, time, iter);
		} else if (animType == 7) {
			return drawShapeCrounched(size, time, iter);
		}
		return null;
	}

	private Shape drawShapeStill(float size, int iter) {
		Shape shape = new Shape();
		float lParam = scaling.getL();
		float footWidth = ((float) 1 / (float) number);// width of a foot
		float xPos = ((float) iter / (float) number - (float) 0.5) * 2
				+ footWidth;
		// xPos is a number from -1 to 1
		float yPos = 1;
		if (shapeType == 0) {
			shape.add(xPos * convergeance, base);
			shape.add(xPos - footWidth / 2, yPos);
			shape.add(xPos + footWidth * lParam, yPos);
		} else if (shapeType == 1) {
			shape.add(xPos * convergeance, base);
			shape.add(xPos * convergeance + footWidth * lParam, base);
			shape.add(xPos * convergeance + footWidth * lParam, yPos);
			shape.add(xPos * convergeance, yPos);
		}
		return shape;
	}

	private Shape drawShapeCrounched(float size, float time, int iter) {
		float orig = Math.abs(time - 0.5f) * 2;// amount of the original sprite;
		Shape shape = new Shape();
		float lParam = scaling.getL();
		float footWidth = ((float) 1 / (float) number);// width of a foot
		float xPos = ((float) iter / (float) number - (float) 0.5) * 2
				+ footWidth;
		// xPos is a number from -1 to 1
		xPos *= (1 - orig) / 2 + 1;
		float yPos = 1;
		float base = this.base + (1 - orig) / 2;// yPos of the upper position
		if (shapeType == 0) {
			shape.add(xPos * convergeance, base);
			shape.add(xPos - footWidth / 2, yPos);
			shape.add(xPos + footWidth * lParam, yPos);
		} else if (shapeType == 1) {
			shape.add(xPos * convergeance, base);
			shape.add(xPos * convergeance + footWidth * lParam, base);
			shape.add(xPos * convergeance + footWidth * lParam, yPos);
			shape.add(xPos * convergeance, yPos);
		}
		return shape;
	}

	public float getConvergeance() {
		return convergeance;
	}

	public void setConvergeance(float convergeance) {
		this.convergeance = convergeance;
	}

	public float getBase() {
		return base;
	}

	public void setBase(float base) {
		this.base = base;
	}

	private Shape drawShapeJumping(float size, float time, int iter) {
		float orig = Math.abs(time - 0.5f) * 2;// amount of the original sprite;
		Shape shape = new Shape();
		float lParam = scaling.getL();
		float footWidth = ((float) 1 / (float) number);// width of a foot
		float xPos = ((float) iter / (float) number - (float) 0.5) * 2
				+ footWidth;
		// xPos is a number from -1 to 1
		xPos *= (orig + 1) / 2;
		float yPos = 1;
		float base = this.base + (orig - 1) / 2;// yPos of the upper position
		if (shapeType == 0) {
			shape.add(xPos * convergeance, base);
			shape.add(xPos - footWidth / 2, yPos);
			shape.add(xPos + footWidth * lParam, yPos);
		} else if (shapeType == 1) {
			shape.add(xPos * convergeance, base);
			shape.add(xPos * convergeance + footWidth * lParam, base);
			shape.add(xPos * convergeance + footWidth * lParam, yPos);
			shape.add(xPos * convergeance, yPos);
		}
		return shape;
	}

	private Shape drawShapeWalking(float size, float time, int iter) {
		Shape shape = new Shape();
		float lParam = scaling.getL();
		float footWidth = ((float) 1 / (float) number);// width of a foot
		float xPos = ((float) iter / (float) number - (float) 0.5) * 2
				+ footWidth;
		// xPos is a number from -1 to 1
		float TWO_PI = PConstants.TWO_PI;
		xPos += PApplet.cos(time * TWO_PI
				+ ((float) iter * TWO_PI * 2 / number))
				/ (number * 3);
		float yPos = 1
				+ PApplet.sin(time * TWO_PI
						+ ((float) iter * TWO_PI * 2 / number)) / (number * 3);
		if (shapeType == 0) {
			shape.add(xPos * convergeance, base);
			shape.add(xPos - footWidth / 2, yPos);
			shape.add(xPos + footWidth * lParam, yPos);
		} else if (shapeType == 1) {
			shape.add(xPos * convergeance, 0);
			shape.add(xPos * convergeance + footWidth * lParam, base);
			shape.add(xPos * convergeance + footWidth * lParam, yPos);
			shape.add(xPos * convergeance, yPos);
		}
		return shape;
	}

	public void randomSet() {
		super.randomSet();
		base = (float) (Math.random() * 0.2);
		convergeance = (float) (Math.random() * 1);
		shapeType = (int) Math.floor(Math.random() * 2);
	}

}
