import java.util.Vector;

import processing.core.PConstants;
import processing.core.PGraphics;
import processing.core.PApplet;

public abstract class Limbs {

	public ColorParam getColor() {
		return color;
	}

	public ScalingParam getScaling() {
		return scaling;
	}

	public void setScaling(ScalingParam scaling) {
		this.scaling = scaling;
	}

	public Movement getMovement() {
		return movement;
	}

	public void setMovement(Movement movement) {
		this.movement = movement;
	}

	public int getShapeType() {
		return shapeType;
	}

	public void setShapeType(int shapeType) {
		this.shapeType = shapeType;
	}

	public void setColor(ColorParam color) {
		this.color = new ColorParam(color);
	}

	public OutlineParam getOutline() {
		return outline;
	}

	public void setOutline(OutlineParam outline) {
		this.outline = new OutlineParam(outline);
	}

	PApplet parent;
	ScalingParam scaling;
	ColorParam color;
	OutlineParam outline;
	Movement movement;
	int shapeType;// in case there are several shapes available
	PGraphics render;

	Shape[] mainShape = new Shape[2];// the second one is only usefull for wings

	Limbs(PApplet p) {
		this.color = new ColorParam();
		this.outline = new OutlineParam();
		this.parent = p;
		this.scaling = new ScalingParam();
		this.movement = new Movement();
		DefaultValues.loadLimb(this);
	}

	protected void beginRender(float w, float h) {
		render = parent.createGraphics((int) w, (int) h, PConstants.JAVA2D);
		float averScale = (scaling.getW() * w + scaling.getH() * h) / 2;
		render.beginDraw();
		render.colorMode(PConstants.ARGB);
		render.smooth();
		render.pushMatrix();
		render.translate(w / 2, h / 2);
		render.translate(scaling.getX() * w / 2, scaling.getY() * h / 2);
		render.scale(scaling.getW() * w, scaling.getH() * h);
		render.scale(scaling.getS());
		render.rotate(scaling.getR());
		render.strokeWeight(outline.getStrokeWidth() / averScale);
		if (!outline.isStrokeVisible()) {
			render.noStroke();
		}
		render.stroke(color.getContrasted(outline.getStrokeContrast()));
		render.fill(color.getARGBInt());
	}

	protected void endRender() {
		render.popMatrix();
		render.endDraw();
	}

	public void randomSet() {
		scaling.randomScale();
		color.randomWalk();
		outline.randomWalk();
		movement.randomSet();
	}

	protected void drawVertexes(Shape shape) {
		for (int i = 0; i < shape.size(); i++) {
			render.vertex(shape.getX(i), shape.getY(i));
		}
		if (shape.size() > 0) {
			render.vertex(shape.getX(0), shape.getY(0));
		}
	}

	protected void drawVertexesShadow(Shape shape, Shape toMatch, float xSh,
			float ySh) {
		// draws a shape inside another one
		toMatch.translate(-xSh, -ySh);
		for (int i = 0; i < shape.size(); i++) {
			if (shape.isIn(i, toMatch)) {
				render.vertex(shape.getX(i), shape.getY(i));
			} else {
				Shape closest = shape.closest(i, toMatch);
				render.vertex(closest.getX(0), closest.getY(0));
			}
		}
		if (shape.size() > 0) {
			if (shape.isIn(0, toMatch)) {
				render.vertex(shape.getX(0), shape.getY(0));
			} else {
				Shape closest = shape.closest(0, toMatch);
				render.vertex(closest.getX(0), closest.getY(0));
			}
		}
		toMatch.translate(xSh, ySh);
	}

	public String[] getSaveStrings() {
		Vector<String> result = new Vector<String>();
		result = MyUtils.addStringArray(result, scaling.getSaveStrings());
		result = MyUtils.addStringArray(result, color.getSaveStrings());
		result = MyUtils.addStringArray(result, outline.getSaveStrings());
		result = MyUtils.addStringArray(result, movement.getSaveStrings());
		result.add(PApplet.str(shapeType));
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadSaveStrings(Vector<String> toLoad) {
		toLoad = scaling.loadSaveStrings(toLoad);
		toLoad = color.loadSaveStrings(toLoad);
		toLoad = outline.loadSaveStrings(toLoad);
		toLoad = movement.loadSaveStrings(toLoad);
		shapeType = Integer.parseInt(toLoad.remove(0));
		return toLoad;
	}

}
