import java.util.Vector;
import processing.core.PApplet;
import processing.core.PConstants;
import processing.core.PGraphics;

public class MainBody {

	public ScalingParam getScaling() {
		return scaling;
	}

	public void setScaling(ScalingParam scaling) {
		this.scaling = scaling;
	}

	public Constitution getConstitution() {
		return constitution;
	}

	public void setConstitution(Constitution constitution) {
		this.constitution = constitution;
	}

	public SpriteSheet getSpriteSheet() {
		return spriteSheet;
	}

	public void setSpriteSheet(SpriteSheet spriteSheet) {
		this.spriteSheet = spriteSheet;
	}

	PApplet parent;
	ColorParam color;// the main color of it
	OutlineParam outline;
	ScalingParam scaling;

	int width;
	int height;

	Constitution constitution;
	SpriteSheet spriteSheet;

	public ColorParam getColor() {
		return new ColorParam(color);
	}

	public void setColor(ColorParam color) {
		this.color = color;
	}

	public OutlineParam getOutline() {
		return new OutlineParam(outline);
	}

	public void setOutline(OutlineParam outline) {
		this.outline = outline;
	}

	MainBody(PApplet p) {
		parent = p;
		this.color = new ColorParam();
		this.outline = new OutlineParam();
		this.constitution = new Constitution(parent, this);
		this.scaling = new ScalingParam();
		DefaultValues.loadMainBody(this);
		this.spriteSheet = new SpriteSheet(parent, this);
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public PGraphics displayStill() {
		return displayFor(0, 0);
	}

	public int getWidth() {
		return (int) ((float) scaling.getW() * width);
	}

	public int getHeight() {
		return (int) ((float) scaling.getH() * height);
	}

	public PGraphics displayFor(int animation, float frame) {
		int w = (int) Math.max(scaling.getW() * width, 1);
		int h = (int) Math.max(scaling.getH() * height, 1);
		PGraphics render = parent.createGraphics(w, h, PConstants.JAVA2D);
		render.beginDraw();
		render.pushMatrix();
		render.translate(scaling.getX() * width, scaling.getY() * height);
		render.scale(scaling.getS());
		render.rotate(scaling.getR());
		render.image(constitution.displayAnim(w, h, frame, animation), 0, 0);
		render.popMatrix();
		render.endDraw();
		return render;
	}

	public PGraphics displaySheet() {
		return spriteSheet.display();
	}

	public void updateSheet() {
		spriteSheet.updateRendered();
	}

	public void updateSheetSingle() {
		spriteSheet.updateRenderedSingle();
	}

	public void saveSheet(String name) {
		spriteSheet.saveSheet(name);
		saveMonsterFile(name);
	}

	public void saveMonsterFile(String name) {
		Vector<String> result = new Vector<String>();
		result.add(PApplet.str(width));
		result.add(PApplet.str(height));
		result = MyUtils.addStringArray(result, scaling.getSaveStrings());
		result = MyUtils.addStringArray(result, spriteSheet.getSaveStrings());
		result = MyUtils.addStringArray(result, constitution.getSaveStrings());
		String[] result2 = MyUtils.toStrArray(result);
		parent.saveStrings(parent.dataPath(name + ".mon"), result2);
	}

	public void loadMonsterFile(String url) {
		String[] strToLoad = parent.loadStrings(url);
		Vector<String> toLoad = new Vector<String>();
		for (int i = 0; i < strToLoad.length; i++) {
			toLoad.add(strToLoad[i]);
		}
		width = Integer.parseInt(toLoad.remove(0));
		height = Integer.parseInt(toLoad.remove(0));
		toLoad = scaling.loadSaveStrings(toLoad);
		toLoad = spriteSheet.loadSaveStrings(toLoad);
		toLoad = constitution.loadSaveStrings(toLoad);
		updateSheet();
	}

	public void addPatchable(int i) {
		constitution.addPatchable(i);
	}

	public void extendParameters() {
		constitution.extendParameters(color, outline);
		updateSheet();
	}

	public void randomMonster() {
		color.randomSet();
		outline.randomSet();
		extendParameters();
		constitution.randomSet();
		updateSheetSingle();
	}

	public void clearLimbs() {
		constitution.clearLimbs();
	}

	public void resetUniqueLimb(int i) {
		constitution.resetUniqueLimb(i);
	}

	public void loadPreset(int i) {
		if (i == 0) {
			clearLimbs();
			resetUniqueLimb(1);
			resetUniqueLimb(2);
			resetUniqueLimb(3);
			addPatchable(2);
			addPatchable(2);
			constitution.getPLimb().get(0).getScaling().setX(-0.3f);
			constitution.getPLimb().get(1).getScaling().setX(+0.3f);
			constitution.getPLimb().get(0).getMovement().setAmount(0.5f);
			constitution.getPLimb().get(1).getMovement().setAmount(0.5f);
			constitution.getTummy().setShapeType(1);
			color.setAHSB((byte) 0xFF, 0.2f, 0.5f, 0.4f);
			extendParameters();
		}
		if (i == 1) {
			clearLimbs();
			resetUniqueLimb(0);
			int nbTent = 10;
			for (int i2 = 0; i2 < nbTent; i2++) {
				addPatchable(3);
				constitution.getPLimb().get(i2).getScaling().setW(0.2f);
				constitution.getPLimb().get(i2).getScaling().setH(0.3f);
				constitution.getPLimb().get(i2).getScaling().setR(
						PConstants.PI / 2 + (float) i2 * PConstants.TWO_PI
								/ nbTent);
				constitution.getPLimb().get(i2).getScaling()
						.setX(
								-PApplet.cos((float) i2 * PConstants.TWO_PI
										/ nbTent) / 3);
				constitution.getPLimb().get(i2).getScaling()
						.setY(
								-PApplet.sin((float) i2 * PConstants.TWO_PI
										/ nbTent) / 3);
			}
			constitution.getSpikes().setComplete(true);
			constitution.getSpikes().setDensity(1);
			constitution.getSpikes().getScaling().setS(1.2f);
			addPatchable(2);
			color.setAHSB((byte) 0xFF, 0.6f, 0.5f, 0.4f);
			extendParameters();
		}
		if (i == 2) {
			clearLimbs();
			resetUniqueLimb(0);
			constitution.getSpikes().setComplete(false);
			constitution.getSpikes().setDensity(3);
			constitution.getSpikes().getScaling().setS(1.2f);
			addPatchable(2);
			addPatchable(2);
			constitution.getPLimb().get(0).getScaling().setX(+0.2f);
			constitution.getPLimb().get(1).getScaling().setX(+0.0f);
			constitution.getPLimb().get(0).getScaling().setY(-0.15f);
			constitution.getPLimb().get(1).getScaling().setY(-0.15f);
			color.setAHSB((byte) 0xFF, 0.1f, 0.5f, 0.4f);
			extendParameters();
		}
		if (i == 3) {
			clearLimbs();
			resetUniqueLimb(0);
			constitution.getSpikes().setComplete(true);
			addPatchable(0);
			constitution.getPLimb().get(0).getScaling().setY(-0.2f);
			addPatchable(2);
			addPatchable(2);
			constitution.getPLimb().get(1).getScaling().setX(-0.15f);
			constitution.getPLimb().get(2).getScaling().setX(+0.15f);
			color.setAHSB((byte) 0xFF, 0.8f, 0.5f, 0.4f);
			extendParameters();
		}
		if (i == 4) {
			clearLimbs();
			resetUniqueLimb(3);
			resetUniqueLimb(1);
			constitution.getTummy().getScaling().setW(0.3f);
			constitution.getFeet().getScaling().setH(0.4f);
			addPatchable(2);
			constitution.pLimb.get(0).getScaling().setY(0.06f);
			constitution.pLimb.get(0).getScaling().setX(0.4f);
			color.setAHSB((byte) 0xFF, 0.3f, 0.5f, 0.4f);
			extendParameters();
		}
		updateSheet();
	}

	public static int getNbPresets() {
		return 5;
	}

	public void removePLimb(int selectedIndex) {
		constitution.pLimb.remove(selectedIndex);
	}

}
