import java.util.Vector;
import processing.core.PApplet;
import processing.core.PGraphics;

abstract public class PatchableLimb extends Limbs {

	PatchableLimb(PApplet p, Constitution constitution) {
		super(p);
		this.bound = new Bound(p, constitution);
	}

	public Bound getBound() {
		return bound;
	}

	public void setBound(Bound bound) {
		this.bound = bound;
	}

	Bound bound;
	public int type;

	protected void setColorVariance() {
		render.stroke(color.getContrasted(outline.getStrokeContrast()));
		render.fill(color.getARGBInt());
		if (!outline.isStrokeVisible()) {
			render.noStroke();
		}
	}

	public int getType() {
		return type;
	}

	public void setType(int type) {
		this.type = type;
	}

	public PGraphics displayStill(float w, float h) {
		beginRender(w, h);
		drawStillRender();
		endRender();
		return render;
	}

	public PGraphics displayAnim(float w, float h, float time, int anim) {
		beginRender(w, h);
		switchAnim(anim, time);
		endRender();
		return render;
	}

	protected void switchAnim(int anim, float time) {
		// 0 : Still 1
		// 1 : Walker walking 24
		// 2 : Walker idling 36
		// 3 : Walker jumping 17
		// 4 : Fly flying 13
		// 5 : Jumper crouch 13
		// 6 : Jumper air 12
		// 7 : Jumper ground 11
		if (anim == 0) {
			drawStillRender();
		} else if (anim == 1) {
			drawWWalkingRender(time);
		} else if (anim == 2) {
			drawWIdlingRender(time);
		} else if (anim == 3) {
			drawWJumpingRender(time);
		} else if (anim == 4) {
			drawFFlyingRender(time);
		} else if (anim == 5) {
			drawJCrounchRender(time);
		} else if (anim == 6) {
			drawJAirRender(time);
		} else if (anim == 7) {
			drawJgroundRender(time);
		}
	}

	abstract protected void drawJgroundRender(float time);

	abstract protected void drawJAirRender(float time);

	abstract protected void drawFFlyingRender(float time);

	abstract protected void drawWJumpingRender(float time);

	abstract protected void drawWIdlingRender(float time);

	abstract protected void drawWWalkingRender(float time);

	abstract protected void drawJCrounchRender(float time);

	abstract protected void drawStillRender();

	public void randomSet() {
		super.randomSet();
		scaling.randomTrans();
		bound.randomSet();
	}

	public String[] getSaveStrings() {
		Vector<String> result = new Vector<String>();
		result = MyUtils.addStringArray(result, super.getSaveStrings());
		result = MyUtils.addStringArray(result, bound.getSaveStrings());
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadSaveStrings(Vector<String> toLoad) {
		toLoad = super.loadSaveStrings(toLoad);
		toLoad = bound.loadSaveStrings(toLoad);
		return toLoad;
	}
}
