import java.util.Vector;

import processing.core.PApplet;
import processing.core.PConstants;
import processing.core.PGraphics;

public class SpikesULimb extends UniqueLimb {

	float density;// number of spikes i each rows
	boolean complete;// false = 180, true = 360

	// shape = triangles, sinusoids, Perlin noise, semi cycles

	protected String[] getAdditionnalSaveStrings() {
		Vector<String> result = new Vector<String>();
		result.add(PApplet.str(density));
		result.add(PApplet.str(complete));
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadAdditionnalSaveStrings(Vector<String> toLoad) {
		density = Float.parseFloat(toLoad.remove(0));
		complete = Boolean.parseBoolean(toLoad.remove(0));
		return toLoad;
	}

	public float getDensity() {
		return density;
	}

	public void setDensity(float density) {
		this.density = density;
	}

	public boolean isComplete() {
		return complete;
	}

	public void setComplete(boolean complete) {
		this.complete = complete;
	}

	public SpikesULimb(PApplet p) {
		super(p);
		DefaultValues.loadSpikes(this);
	}

	public PGraphics displayAnim(float w, float h, float time, int anim) {
		beginRender(w, h);
		for (int i = 0; i < number; i++) {
			setColorVariance(i);
			if (i != 0)
				render.noStroke();
			drawShape(anim, i, time);
		}
		setColorVariance(0);
		render.noFill();
		drawShapeWithoutShadow(anim, 0, time);
		endRender();
		return render;
	}

	protected void drawShape(int animType, int iter, float time) {
		drawShapeWithoutShadow(animType, iter, time);
		drawShadow(animType, iter, time);
	}

	protected void drawShapeWithoutShadow(int animType, int iter, float time) {
		float diameter = (float) (number - iter) / number;
		render.beginShape();
		drawVertexes(switchAnim(animType, iter, time, diameter));
		if (iter == 0) {
			mainShape[0] = switchAnim(animType, iter, time, diameter);
		}
		render.endShape();
	}

	protected void drawShadow(int animType, int iter, float time) {
		float diameter = (float) (number - iter) / number;
		setShadowColor(iter);
		diameter *= outline.getShadowLevel();
		render.pushMatrix();
		render.beginShape();
		// shifting of the shadow
		float xSh = (float) Math.cos(outline.getShadowPosition())
				* outline.getShadowDistance() / 3;
		float ySh = (float) Math.sin(outline.getShadowPosition())
				* outline.getShadowDistance() / 3;
		render.translate(xSh, ySh);
		drawVertexesShadow(switchAnim(animType, iter, time, diameter),
				mainShape[0], xSh, ySh);
		render.endShape();
		render.popMatrix();
	}

	Shape switchAnim(int animType, int iter, float time, float diameter) {
		if (animType == 0) {
			return drawShapeStill(diameter, iter);
		} else if (animType == 1) {
			return drawShapeWalking(diameter, time, iter);
		} else if (animType == 2) {
			return drawShapeIdling(diameter, time, iter);
		} else if (animType == 3) {
			return drawShapeJumping(diameter, time, iter);
		} else if (animType == 4) {
			return drawShapeIdling(diameter, time, iter);
		} else if (animType == 5) {
			return drawShapeCrounched(diameter, time, iter);
		} else if (animType == 6) {
			return drawShapeIdling(diameter, time, iter);
		} else if (animType == 7) {
			return drawShapeCrounched(diameter, time, iter);
		}
		return null;
	}

	private Shape drawOneSpike(float lMin, float lMax, float aMin, float aMax,
			float phase) {
		// phase is from 0 to 1
		Shape shape = new Shape();
		if (complete) {
			phase = (1 - ((phase * (density / 3)) % 1));
		}
		if (shapeType == 0) {
			int divs = 7;
			for (int i = 0; i < divs; i++) {
				float a = aMin + (aMax - aMin) * (float) i / (float) divs;
				float l = lMin
						+ (lMax - lMin)
						* (float) (Math.abs(((i + phase * divs) % divs)
								- (float) divs / 2) * 2 / divs);
				float xP = (float) (Math.cos(a) * l);
				float yP = (float) (Math.sin(a) * l);
				shape.add(xP, yP);
			}
		} else if (shapeType >= 1) {
			int divs = 7;
			for (int i = 0; i < divs; i++) {
				float a = aMin + (aMax - aMin) * (float) i / (float) divs;
				float l = lMin
						+ (lMax - lMin)
						* PApplet.sin(i * PApplet.TWO_PI / divs + phase
								* PConstants.TWO_PI);
				float xP = (float) (Math.cos(a) * l);
				float yP = (float) (Math.sin(a) * l);
				shape.add(xP, yP);
			}
		}
		return shape;
	}

	private Shape drawShapeCrounched(float diameter, float time, int iter) {
		Shape shape = new Shape();
		float lParam = scaling.getL();
		Shape extremities = new Shape();
		for (int i2 = 0; i2 < density; i2++) {
			float lMin = lParam * diameter;
			float PI = PConstants.PI;
			float angle = (float) i2 / (float) density;
			float diameter2;
			if (complete) {
				diameter2 = diameter * (PApplet.abs(angle - 0.5f) * 2) * 1.5f;
			} else {
				diameter2 = diameter * (PApplet.abs(angle - 0.5f) * 2) * 1.5f;
			}
			float lMax = diameter * (1 - time) + (diameter2 * time);
			float aMin;
			float aMax;
			if (complete) {
				aMin = PI / 2 + ((float) i2 * (PI * 2) / (float) density);
				aMax = PI / 2 + ((float) (i2 + 1) * (PI * 2) / (float) density);
			} else {
				aMin = PI + ((float) i2 * PI / (float) density);
				aMax = PI + ((float) (i2 + 1) * PI / (float) density);
			}
			shape.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0));
			if (i2 == 0) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.get(0));
			}
			if (i2 == density - 1) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.getLast());
			}
		}
		if (!complete) {
			for (int i2 = 0; i2 < density; i2++) {
				float x = extremities.getX(0) * (i2 / density)
						+ extremities.getX(1) * ((density - i2) / density);
				float y = extremities.getY(0) * (i2 / density)
						+ extremities.getY(1) * ((density - i2) / density);
				shape.add(x, y);
			}
		}
		return shape;
	}

	protected Shape drawShapeStill(float diameter, int iter) {
		Shape shape = new Shape();
		float lParam = scaling.getL();
		Shape extremities = new Shape();
		for (int i2 = 0; i2 < density; i2++) {
			float lMin = lParam * diameter;
			float lMax = diameter;
			float aMin;
			float aMax;
			if (complete) {
				float TWO_PI = PConstants.TWO_PI;
				aMin = TWO_PI + ((float) i2 * TWO_PI / (float) density);
				aMax = TWO_PI + ((float) (i2 + 1) * TWO_PI / (float) density);
			} else {
				float PI = PConstants.PI;
				aMin = PI + ((float) i2 * PI / (float) density);
				aMax = PI + ((float) (i2 + 1) * PI / (float) density);
			}
			shape.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0));
			if (i2 == 0) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.get(0));
			}
			if (i2 == density - 1) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.getLast());
			}
		}
		if (!complete) {
			for (int i2 = 0; i2 < density; i2++) {
				float x = extremities.getX(0) * (i2 / density)
						+ extremities.getX(1) * ((density - i2) / density);
				float y = extremities.getY(0) * (i2 / density)
						+ extremities.getY(1) * ((density - i2) / density);
				shape.add(x, y);
			}
		}
		return shape;
	}

	private Shape drawShapeJumping(float diameter, float time, int iter) {
		Shape shape = new Shape();
		float lParam = scaling.getL();
		Shape extremities = new Shape();
		for (int i2 = 0; i2 < density; i2++) {
			float lMin = lParam * diameter;
			float PI = PConstants.PI;
			float angle = (float) i2 / (float) density;
			float diameter2;
			if (complete) {
				diameter2 = diameter * (1 - PApplet.abs(angle - 0.5f) * 2)
						* 1.5f;
			} else {
				diameter2 = diameter * (1 - PApplet.abs(angle - 0.5f) * 2)
						* 1.5f;
			}
			float lMax = diameter * (1 - time) + (diameter2 * time);
			float aMin;
			float aMax;
			if (complete) {
				aMin = PI / 2 + ((float) i2 * (PI * 2) / (float) density);
				aMax = PI / 2 + ((float) (i2 + 1) * (PI * 2) / (float) density);
			} else {
				aMin = PI + ((float) i2 * PI / (float) density);
				aMax = PI + ((float) (i2 + 1) * PI / (float) density);
			}
			shape.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0));
			if (i2 == 0) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.get(0));
			}
			if (i2 == density - 1) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.getLast());
			}
		}
		if (!complete) {
			for (int i2 = 0; i2 < density; i2++) {
				float x = extremities.getX(0) * (i2 / density)
						+ extremities.getX(1) * ((density - i2) / density);
				float y = extremities.getY(0) * (i2 / density)
						+ extremities.getY(1) * ((density - i2) / density);
				shape.add(x, y);
			}
		}
		return shape;
	}

	private Shape drawShapeIdling(float diameter, float time, int iter) {
		Shape shape = new Shape();
		float lParam = scaling.getL();
		Shape extremities = new Shape();
		for (int i2 = 0; i2 < density; i2++) {
			float lMin = lParam * diameter;
			float lMax = diameter;
			float aMin;
			float aMax;
			float TWO_PI = PConstants.TWO_PI;
			if (complete) {
				aMin = TWO_PI + ((float) i2 * TWO_PI / (float) density);
				aMax = TWO_PI + ((float) (i2 + 1) * TWO_PI / (float) density);
			} else {
				float PI = PConstants.PI;
				aMin = PI + ((float) i2 * PI / (float) density);
				aMax = PI + ((float) (i2 + 1) * PI / (float) density);
			}
			float phase = 0;
			if (complete) {
				phase = (PApplet.sin(time * TWO_PI) + 1) / density;
			} else {
				phase = (PApplet.sin(time * TWO_PI) + 1) / 2;
			}
			shape.addShape(drawOneSpike(lMin, lMax, aMin, aMax, phase));
			if (i2 == 0) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.get(0));
			}
			if (i2 == density - 1) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.getLast());
			}
		}
		if (!complete) {
			for (int i2 = 0; i2 < density; i2++) {
				float x = extremities.getX(0) * (i2 / density)
						+ extremities.getX(1) * ((density - i2) / density);
				float y = extremities.getY(0) * (i2 / density)
						+ extremities.getY(1) * ((density - i2) / density);
				shape.add(x, y);
			}
		}
		return shape;
	}

	private Shape drawShapeWalking(float diameter, float time, int iter) {
		Shape shape = new Shape();
		float lParam = scaling.getL();
		Shape extremities = new Shape();
		for (int i2 = 0; i2 < density; i2++) {
			float lMin = lParam * diameter;
			float lMax = diameter;
			float aMin;
			float aMax;
			if (complete) {
				float TWO_PI = PConstants.TWO_PI;
				aMin = TWO_PI + ((float) i2 * TWO_PI / (float) density);
				aMax = TWO_PI + ((float) (i2 + 1) * TWO_PI / (float) density);
			} else {
				float PI = PConstants.PI;
				aMin = PI + ((float) i2 * PI / (float) density);
				aMax = PI + ((float) (i2 + 1) * PI / (float) density);
			}
			shape.addShape(drawOneSpike(lMin, lMax, aMin, aMax, time));
			if (i2 == 0) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.get(0));
			}
			if (i2 == density - 1) {
				extremities.addShape(drawOneSpike(lMin, lMax, aMin, aMax, 0)
						.getLast());
			}
		}
		if (!complete) {
			for (int i2 = 0; i2 < density; i2++) {
				float x = extremities.getX(0) * (i2 / density)
						+ extremities.getX(1) * ((density - i2) / density);
				float y = extremities.getY(0) * (i2 / density)
						+ extremities.getY(1) * ((density - i2) / density);
				shape.add(x, y);
			}
		}
		return shape;
	}

	public void randomSet() {
		super.randomSet();
		density = (float) (Math.floor(Math.random() * 15) + 1);
		if (Math.random() < 0.5f) {
			complete = true;
		} else {
			complete = false;
		}
		shapeType = (int) Math.floor(Math.random() * 2);
	}

	public void switchComplete() {
		if (complete) {
			complete = false;
		} else {
			complete = true;
		}
	}
}
