import java.util.Vector;

import processing.core.PApplet;
import processing.core.PConstants;
import processing.core.PGraphics;

public class SpriteSheet {

	MainBody mainBody;
	PApplet parent;
	PGraphics rendered;
	PGraphics[] sprite;
	int spriteWidth, spriteHeight;
	Animation animation;
	int rawSize = 10;// how many sprites on one raw

	public PGraphics[] getSprite() {
		return sprite;
	}

	public void setSprite(PGraphics[] sprite) {
		this.sprite = sprite;
	}

	public int getSpriteWidth() {
		return spriteWidth;
	}

	public void setSpriteWidth(int spriteWidth) {
		this.spriteWidth = spriteWidth;
	}

	public int getSpriteHeight() {
		return spriteHeight;
	}

	public void setSpriteHeight(int spriteHeight) {
		this.spriteHeight = spriteHeight;
	}

	public Animation getAnimation() {
		return animation;
	}

	public void setAnimation(Animation animation) {
		this.animation = animation;
	}

	SpriteSheet(PApplet p, MainBody mainBody) {
		this.parent = p;
		this.mainBody = mainBody;
		this.spriteWidth = mainBody.getWidth();
		this.spriteHeight = mainBody.getHeight();
		rendered = parent.createGraphics(parent.width, parent.height,
				PConstants.JAVA2D);
		this.animation = DefaultValues.animation();
	}

	public void updateRendered() {
		int pW = parent.width - (parent.width % spriteWidth);
		int pH = parent.height - (parent.height % spriteHeight);
		PGraphics render = parent.createGraphics(pW, pH, PConstants.JAVA2D);
		render.beginDraw();
		int frameNb = 0;
		for (int i = 0; i < animation.getNbFrames().length; i++) {
			if (animation.getNbFrames()[i] > 0) {
				for (int i2 = 0; i2 < animation.getNbFrames()[i]; i2++) {
					render.pushMatrix();
					int[] trans = spreadSprites(frameNb, spriteWidth,
							spriteHeight);
					render.translate(trans[0], trans[1]);
					float time = (float) i2
							/ (float) animation.getNbFrames()[i];
					// time is the current frame,
					// considering the animation goes from 0 to 1
					render.image(mainBody.displayFor(i, time), 0, 0);
					render.popMatrix();
					frameNb++;
				}
			}
		}
		render.endDraw();
		rendered = parent.createGraphics(pW, pH, PConstants.JAVA2D);
		rendered = render;
	}

	public void updateRenderedSingle() {
		int pW = parent.width - (parent.width % spriteWidth);
		int pH = parent.height - (parent.height % spriteHeight);
		PGraphics render = parent.createGraphics(pW, pH, PConstants.JAVA2D);
		render.beginDraw();
		render.image(mainBody.displayStill(), 0, 0);
		render.endDraw();
		rendered = parent.createGraphics(pW, pH, PConstants.JAVA2D);
		rendered = render;
	}

	private int[] spreadSprites(int fNb, int sW, int sH) {
		int[] values = new int[2];
		int placeX = (int) (fNb % Math.floor(parent.width / sW));
		int placeY = (int) (fNb / Math.floor(parent.width / sW));
		values[0] = placeX * sW;
		values[1] = placeY * sH;
		return values;
	}

	private int[] spreadSpritesIn(int fNb, int sW, int sH, int cWidth) {
		int[] values = new int[2];
		int placeX = (int) (fNb % Math.floor(cWidth / sW));
		int placeY = (int) (fNb / Math.floor(cWidth / sW));
		values[0] = placeX * sW;
		values[1] = placeY * sH;
		return values;
	}

	public PGraphics displayGrid() {
		int pW = parent.width;
		int pH = parent.height;
		PGraphics render = parent.createGraphics(pW, pH, PConstants.JAVA2D);
		render.beginDraw();
		render.colorMode(PConstants.ARGB);
		render.stroke(0xFFFF0000);
		render.strokeWeight(1);
		for (int x = 0; x <= pW; x += spriteWidth) {
			render.line(x, 0, x, pH);
		}
		for (int y = 0; y <= pH; y += spriteHeight) {
			render.line(0, y, pW, y);
		}
		render.endDraw();
		return render;
	}

	public PGraphics display() {
		return rendered;
	}

	public void saveSheet(String name) {
		updateRealSheet();
		System.out.println(parent.dataPath(name + "_" + spriteWidth + "_"
				+ spriteHeight + ".png"));
		rendered.save(parent.dataPath(name + "_" + spriteWidth + "_"
				+ spriteHeight + ".png"));
	}

	private void updateRealSheet() {
		rawSize = findBestRawSize();
		int pW = rawSize * spriteWidth;
		int pH = (int) (spriteHeight * Math.ceil((float) animation
				.getTotalFrames()
				/ rawSize));
		PGraphics render = parent.createGraphics(pW, pH, PConstants.JAVA2D);
		render.beginDraw();
		int frameNb = 0;
		for (int i = 0; i < animation.getNbFrames().length; i++) {
			if (animation.getNbFrames()[i] > 0) {
				for (int i2 = 0; i2 < animation.getNbFrames()[i]; i2++) {
					render.pushMatrix();
					int[] trans = spreadSpritesIn(frameNb, spriteWidth,
							spriteHeight, pW);
					render.translate(trans[0], trans[1]);
					float time = (float) i2
							/ (float) animation.getNbFrames()[i];
					// time is the current frame,
					// considering the animation goes from 0 to 1
					render.image(mainBody.displayFor(i, time), 0, 0);
					render.popMatrix();
					frameNb++;
				}
			}
		}
		render.endDraw();
		rendered = parent.createGraphics(pW, pH, PConstants.JAVA2D);
		rendered = render;
	}

	public void findBestSize() {
		mainBody.getScaling().setX(0);
		mainBody.getScaling().setY(0);
		spriteWidth = PApplet.max(mainBody.getWidth(), 1);
		spriteHeight = PApplet.max(mainBody.getHeight(), 1);
		updateRealSheet();
		int minX = spriteWidth;// the first tile where there's something to see
		int minY = spriteHeight;// the first tile where there's something to see
		int maxX = 0;// the last tile where there's something to see
		int maxY = 0;// the last tile where there's something to see
		rendered.updatePixels();
		for (int i = 0; i < animation.getTotalFrames(); i++) {
			int frameX = (i * spriteWidth) % rendered.width;
			int frameY = (i / (rendered.width / spriteWidth) * spriteHeight);
			for (int x = frameX; x < frameX + spriteWidth; x++) {
				for (int y = frameY; y < frameY + spriteHeight; y++) {
					// if the alpha channel is > 0
					if ((rendered.get(x, y) >> 24 & 0xFF) > 0) {
						minX = Math.min(minX, x - frameX);
						maxX = Math.max(maxX, x - frameX);
						minY = Math.min(minY, y - frameY);
						maxY = Math.max(maxY, y - frameY);
					}
				}
			}
		}
		// + 2 and - 1 are for the border of tiles
		spriteWidth = maxX - minX + 2;
		spriteHeight = maxY - minY + 2;
		mainBody.getScaling().setX(
				(float) -(minX - 1) / ((float) mainBody.getWidth()));
		mainBody.getScaling().setY(
				(float) -(minY - 1) / ((float) mainBody.getHeight()));
	}

	public int findBestRawSize() {
		int bestI = -1;// best number in one raw
		int bestTotal = -1;// best number of total pixels
		int tF = animation.getTotalFrames();
		for (int i = 1; i < Math.ceil(tF / 2); i++) {
			int xSize = (spriteWidth * i);
			int ySize = (spriteHeight * (int) Math.ceil((float) tF / (float) i));
			int xSizeP = 0;// next power of two
			int ySizeP = 0;// next power of two
			for (int i2 = 1; xSize > xSizeP; i2++) {
				xSizeP = (int) Math.pow(2, i2);
			}
			for (int i2 = 1; ySize > ySizeP; i2++) {
				ySizeP = (int) Math.pow(2, i2);
			}
			int totalPixels = xSizeP * ySizeP;
			if (bestTotal == -1 || totalPixels <= bestTotal) {
				bestI = i;
				bestTotal = totalPixels;
			}
		}
		return bestI;
	}

	public String[] getSaveStrings() {
		Vector<String> result = new Vector<String>();
		result.add(PApplet.str(spriteWidth));
		result.add(PApplet.str(spriteHeight));
		result = MyUtils.addStringArray(result, animation.getSaveStrings());
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadSaveStrings(Vector<String> toLoad) {
		spriteWidth = Integer.parseInt(toLoad.remove(0));
		spriteHeight = Integer.parseInt(toLoad.remove(0));
		toLoad = animation.loadSaveStrings(toLoad);
		return toLoad;
	}
}
