import java.util.Vector;

import processing.core.PApplet;
import processing.core.PConstants;
import processing.core.PGraphics;

public class TummyULimb extends UniqueLimb {

	int divs;// number of vertex
	float irreg;// default values

	protected String[] getAdditionnalSaveStrings() {
		Vector<String> result = new Vector<String>();
		result.add(PApplet.str(divs));
		result.add(PApplet.str(irreg));
		String[] result2 = MyUtils.toStrArray(result);
		return result2;
	}

	public Vector<String> loadAdditionnalSaveStrings(Vector<String> toLoad) {
		divs = Integer.parseInt(toLoad.remove(0));
		irreg = Float.parseFloat(toLoad.remove(0));
		return toLoad;
	}

	public TummyULimb(PApplet p) {
		super(p);
		DefaultValues.loadTummy(this);
	}

	public int getDivs() {
		return divs;
	}

	public void setDivs(int divs) {
		this.divs = divs;
	}

	public float getIrreg() {
		return irreg;
	}

	public void setIrreg(float irreg) {
		this.irreg = irreg;
	}

	public PGraphics displayAnim(float w, float h, float time, int anim) {
		beginRender(w, h);
		for (int i = 0; i < number; i++) {
			setColorVariance(i);
			if (i != 0)
				render.noStroke();
			drawShape(anim, i, time);
		}
		setColorVariance(0);
		render.noFill();
		drawShapeWithoutShadow(anim, 0, time);
		endRender();
		return render;
	}

	protected void drawShape(int animType, int iter, float time) {
		drawShapeWithoutShadow(animType, iter, time);
		drawShadow(animType, iter, time);
	}

	protected void drawShapeWithoutShadow(int animType, int iter, float time) {
		float diameter = (float) (number - iter) / number;
		render.beginShape();
		drawVertexes(switchAnim(animType, iter, time, diameter));
		if (iter == 0) {
			mainShape[0] = switchAnim(animType, iter, time, diameter);
		}
		render.endShape();
	}

	protected void drawShadow(int animType, int iter, float time) {
		float diameter = (float) (number - iter) / number;
		setShadowColor(iter);
		diameter *= outline.getShadowLevel();
		render.pushMatrix();
		render.beginShape();
		// shifting of the shadow
		float xSh = (float) Math.cos(outline.getShadowPosition())
				* outline.getShadowDistance();
		float ySh = (float) Math.sin(outline.getShadowPosition())
				* outline.getShadowDistance();
		render.translate(xSh, ySh);
		drawVertexesShadow(switchAnim(animType, iter, time, diameter),
				mainShape[0], xSh, ySh);
		render.endShape();
		render.popMatrix();
	}

	Shape switchAnim(int animType, int iter, float time, float diameter) {
		if (animType == 0) {
			return drawShapeStill(diameter, iter);
		} else if (animType == 1) {
			return drawShapeWalking(diameter, time, iter);
		} else if (animType == 2) {
			return drawShapeIdling(diameter, time, iter);
		} else if (animType == 3) {
			return drawShapeJumping(diameter, time, iter);
		} else if (animType == 4) {
			return drawShapeFlying(diameter, time, iter);
		} else if (animType == 5) {
			return drawShapeCrounched(diameter, time, iter);
		} else if (animType == 6) {
			return drawShapeFlying(diameter, time, iter);
		} else if (animType == 7) {
			return drawShapeCrounched(diameter, time, iter);
		}
		return null;
	}

	private Shape drawShapeStill(float diameter, int iter) {
		Shape shape = new Shape();
		float TWO_PI = PConstants.TWO_PI;
		for (int i2 = 0; i2 < divs; i2++) {
			float diameterV = diameter;// including the shapeType variation
			if (shapeType == 1) {
				diameterV *= ((i2 % 2) * irreg + 1);
			}
			float x = (float) (Math.cos(i2 * TWO_PI / divs) * diameterV);
			float y = (float) (Math.sin(i2 * TWO_PI / divs) * diameterV);
			shape.add(x, y);
		}
		return shape;
	}

	private Shape drawShapeFlying(float diameter, float time, float iter) {
		Shape shape = new Shape();
		float TWO_PI = PConstants.TWO_PI;
		for (int i2 = 0; i2 < divs; i2++) {
			float angle = (float) i2 / (float) divs;
			float diameterV = diameter;// including the shapeType variation
			if (shapeType == 1) {
				diameterV *= ((i2 % 2) * irreg + 1);
			}
			float diameter2 = diameterV
					* (float) (Math.cos(angle * TWO_PI * 2) + 3) / 3;
			diameter2 += (float) Math.sin(angle * TWO_PI) / 4
					* (float) Math.sin(time * TWO_PI) * 2;
			// set the correct amount of it sepending on the Movement object
			diameter2 = (diameter2 * movement.getAmount())
					+ (diameterV * (1 - movement.getAmount()));			
			float x = (float) (Math.cos(i2 * TWO_PI / divs));
			x *= (diameter2 * (0.5 - Math.abs(time - 0.5)) * 2 + diameterV
					* Math.abs(time - 0.5) * 2);
			float y = (float) (Math.sin(i2 * TWO_PI / divs));
			y *= (diameter2 * (0.5 - Math.abs(time - 0.5)) * 2 + diameterV
					* Math.abs(time - 0.5) * 2);
			y += (time * iter / number) / 2;
			y += (float) Math.sin(time * TWO_PI) / 10;
			shape.add(x, y);
		}
		return shape;
	}

	private Shape drawShapeCrounched(float diameter, float time, float iter) {
		Shape shape = new Shape();
		float TWO_PI = PConstants.TWO_PI;
		for (int i2 = 0; i2 < divs; i2++) {
			float angle = (float) i2 / (float) divs;
			float diameterV = diameter;// including the shapeType variation
			if (shapeType == 1) {
				diameterV *= ((i2 % 2) * irreg + 1);
			}
			float diameter2 = diameterV
					* (float) (Math.cos(angle * TWO_PI * 2) + 2) / 2;
			diameter2 += (float) Math.sin(angle * TWO_PI) / 4;
			float x = (float) (Math.cos(i2 * TWO_PI / divs));
			// set the correct amount of it sepending on the Movement object
			diameter2 = (diameter2 * movement.getAmount())
					+ (diameterV * (1 - movement.getAmount()));			
			x *= (diameter2 * (1 - Math.abs(time - 0.5f) * 2) + diameterV
					* Math.abs(time - 0.5f) * 2);
			float y = (float) (Math.sin(i2 * TWO_PI / divs));
			y *= (diameter2 * (1 - Math.abs(time - 0.5f) * 2) + diameterV
					* Math.abs(time - 0.5f) * 2);
			y += (time * iter / number) / 2;
			shape.add(x, y);
		}
		return shape;
	}

	private Shape drawShapeJumping(float diameter, float time, float iter) {
		Shape shape = new Shape();
		float TWO_PI = PConstants.TWO_PI;
		for (int i2 = 0; i2 < divs; i2++) {
			float angle = (float) i2 / (float) divs;
			float diameterV = diameter;// including the shapeType variation
			if (shapeType == 1) {
				diameterV *= ((i2 % 2) * irreg + 1);
			}
			float diameter2 = diameterV
					* (float) (-Math.cos(angle * TWO_PI * 2) + 2) / 2;
			diameter2 += (float) Math.sin(angle * TWO_PI) / 4;
			// set the correct amount of it sepending on the Movement object
			diameter2 = (diameter2 * movement.getAmount())
					+ (diameterV * (1 - movement.getAmount()));			
			float x = (float) (Math.cos(i2 * TWO_PI / divs));
			x *= (diameter2 * time + diameterV * (1 - time));
			float y = (float) (Math.sin(i2 * TWO_PI / divs));
			y *= (diameter2 * time + diameterV * (1 - time));
			y += (time * iter / number) / 2;
			shape.add(x, y);
		}
		return shape;
	}

	private Shape drawShapeWalking(float diameter, float time, float iter) {
		Shape shape = new Shape();
		float TWO_PI = PConstants.TWO_PI;
		for (int i2 = 0; i2 < divs; i2++) {
			float angle = (float) i2 / (float) divs;
			float diameterV = diameter;// including the shapeType variation
			if (shapeType == 1) {
				diameterV *= ((i2 % 2) * irreg + 1);
			}
			float diameter2 = diameterV
					* (float) (Math.cos(angle * TWO_PI * 2) + 3) / 3;
			diameter2 += (float) Math.sin(angle * TWO_PI - 1) / 4
					* (float) Math.sin(time * TWO_PI) * 2;
			float x = (float) (Math.cos(i2 * TWO_PI / divs));
			// set the correct amount of it sepending on the Movement object
			diameter2 = (diameter2 * movement.getAmount())
					+ (diameterV * (1 - movement.getAmount()));
			x *= (diameter2 * (0.5 - Math.abs(time - 0.5)) * 2 + diameterV
					* Math.abs(time - 0.5) * 2);
			float y = (float) (Math.sin(i2 * TWO_PI / divs));
			y *= (diameter2 * (0.5 - Math.abs(time - 0.5)) * 2 + diameterV
					* Math.abs(time - 0.5) * 2);
			y += (time * iter / number) / 2;
			y += (float) Math.sin(time * TWO_PI) / 10;
			shape.add(x, y);
		}
		return shape;
	}

	private Shape drawShapeIdling(float diameter, float time, float iter) {
		Shape shape = new Shape();
		float TWO_PI = PConstants.TWO_PI;
		for (int i2 = 0; i2 < divs; i2++) {
			float angle = (float) i2 / (float) divs;
			float diameterV = diameter;// including the shapeType variation
			if (shapeType == 1) {
				diameterV *= ((i2 % 2) * irreg + 1);
			}
			float diameter2 = diameterV
					* (float) (Math.cos(angle * TWO_PI * 2) + 3) / 3;
			diameter2 += (float) Math.sin(angle * TWO_PI) / 4
					* (float) Math.sin(time * TWO_PI) * 2;
			float x = (float) (Math.cos(i2 * TWO_PI / divs));
			// set the correct amount of it sepending on the Movement object
			diameter2 = (diameter2 * movement.getAmount())
					+ (diameterV * (1 - movement.getAmount()));			
			x *= (diameter2 * (0.5 - Math.abs(time - 0.5)) * 2 + diameterV
					* Math.abs(time - 0.5) * 2);
			float y = (float) (Math.sin(i2 * TWO_PI / divs));
			y *= (diameter2 * (0.5 - Math.abs(time - 0.5)) * 2 + diameterV
					* Math.abs(time - 0.5) * 2);
			y += (time * iter / number) / 2;
			y += (float) -Math.sin(time * TWO_PI) / 5;
			shape.add(x, y);
		}
		return shape;
	}

	public void randomSet() {
		super.randomSet();
		divs = (int) (Math.random() * 50 + 1);
		irreg = (float) Math.random();
		shapeType = (int) Math.floor(Math.random() * 2);
	}

}
